<?php
/*
* 2007-2018 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2011 PrestaShop SA
*  @version  Release: $Revision: 12876 $
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_'))
	exit;

class EmsCarrier extends CarrierModule
{
	public  $id_carrier;
	private $_html = '';
	private $uninstallCalled = false;
	private $_postErrors = array();
	private $_moduleName = 'emscarrier';
	private $configOptions = array(
		'EMS_API_URL',
		'EMS_API_USERNAME',
		'EMS_API_TOKEN',
		'EMS_CHANNEL'
	);

	public function __construct()
	{
		$this->name = 'emscarrier';
		$this->tab = 'shipping_logistics';
		$this->version = '1.0.1';
		$this->author = 'Tristone Cloud Inc.';
		$this->ps_versions_compliancy = array('min' => '1.7.0.0', 'max' => _PS_VERSION_);

		parent::__construct();

		$this->displayName = $this->l('EMarketplaceservices (EMS) live shipping rate calulator');
		$this->description = $this->l('Calculates actual shipping rate based on produt weight during checkout.');

		$config = Configuration::getMultiple($this->configOptions);

		if (isset($config['EMS_API_URL']))
			$this->api_url = $config['EMS_API_URL'];

		if (isset($config['EMS_API_USERNAME']))
			$this->api_username = $config['EMS_API_USERNAME'];

		if (isset($config['EMS_API_TOKEN']))
			$this->api_token = $config['EMS_API_TOKEN'];

		if (isset($config['EMS_CHANNEL']))
			$this->channel = $config['EMS_CHANNEL'];
	}

	/*
	** Install / Uninstall Methods
	**
	*/

	public function install()
	{
		if (Shop::isFeatureActive())
			Shop::setContext(Shop::CONTEXT_ALL);
		// Install SQL
		include(dirname(__FILE__) . '/sql-install.php');
		foreach ($sql as $s)
			if (!Db::getInstance()->execute($s))
				return false;

		// Install Module
		if (!parent::install() or !$this->registerHook('updateCarrier'))
			return false;

		return true;
	}

	public function uninstall()
	{
		// Uninstall Carriers
		$uninstallCalled = true;
		Db::getInstance()->update('carrier', array('deleted' => 1), '`external_module_name` = \'emscarrier\' OR `id_carrier` IN (SELECT DISTINCT(`id_carrier`) FROM `' . _DB_PREFIX_ . 'ems_rate_service_code`)');

		// Uninstall Config
		foreach ($this->configOptions as $configOption)
			if (!Configuration::deleteByName($configOption))
				return false;

		// Uninstall SQL
		include(dirname(__FILE__) . '/sql-uninstall.php');
		foreach ($sql as $s)
			if (!Db::getInstance()->execute($s))
				return false;

		// Uninstall Module
		if (!parent::uninstall() or !$this->unregisterHook('updateCarrier'))
			return false;

		return true;
	}

	public function disable($forceAll = false)
	{
		// Disable Carriers

		if (!count(Db::getInstance()->executeS("Show tables like '" . _DB_PREFIX_ . 'ems_rate_service_code' . "'"))) {
			return parent::disable($forceAll);
		}

		Db::getInstance()->update('carrier', array('active' => 0), '`external_module_name` = \'emscarrier\' OR `id_carrier` IN (SELECT DISTINCT(`id_carrier`) FROM `' . _DB_PREFIX_ . 'ems_rate_service_code`)');
		return parent::disable($forceAll);
	}

	public function enable($forceAll = false)
	{
		// Disable Carriers
		Db::getInstance()->update('carrier', array('active' => 1), '`external_module_name` = \'emscarrier\' OR `id_carrier` IN (SELECT DISTINCT(`id_carrier`) FROM `' . _DB_PREFIX_ . 'ems_rate_service_code`)');
		return parent::enable($forceAll);
	}

	public function installCarriers()
	{
		// Get all services availables
		$rateServiceList = Db::getInstance()->executeS('SELECT * FROM `' . _DB_PREFIX_ . 'ems_rate_service_code`');

		if (count($rateServiceList)) {
			foreach ($rateServiceList as $rateService)
				if (!$rateService['id_carrier']) {
					if (trim($rateService['transit_time']) == '')
						$rateService['transit_time'] = $rateService['service_name'];
					$config = array(
						'name' => substr($rateService['service_name'] . '(' . $rateService['service_code'] . ')', 0, 64),
						'id_tax_rules_group' => 0,
						'active' => true,
						'deleted' => 0,
						'shipping_handling' => false,
						'range_behavior' => 0,
						'delay' => array('en' => $rateService['transit_time'], 'fr' => $rateService['transit_time'], Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')) => $rateService['transit_time']),
						'id_zone' => 1,
						'is_module' => true,
						'shipping_external' => true,
						'external_module_name' => $this->_moduleName,
						'need_range' => true
					);
					$id_carrier = $this->installExternalCarrier($config);
					Db::getInstance()->update('ems_rate_service_code', array('id_carrier' => (int) ($id_carrier)), '`id_ems_rate_service_code` = ' . (int) ($rateService['id_ems_rate_service_code']));
				}
		}
	}

	public static function installExternalCarrier($config)
	{
		$carrier = new Carrier();
		$carrier->name = $config['name'];
		$carrier->id_tax_rules_group = $config['id_tax_rules_group'];
		$carrier->id_zone = $config['id_zone'];
		$carrier->active = $config['active'];
		$carrier->deleted = $config['deleted'];
		$carrier->delay = $config['delay'];
		$carrier->shipping_handling = $config['shipping_handling'];
		$carrier->range_behavior = $config['range_behavior'];
		$carrier->is_module = $config['is_module'];
		$carrier->shipping_external = $config['shipping_external'];
		$carrier->external_module_name = $config['external_module_name'];
		$carrier->need_range = $config['need_range'];

		$languages = Language::getLanguages(true);
		foreach ($languages as $language) {
			if ($language['iso_code'] == 'fr')
				$carrier->delay[(int) $language['id_lang']] = $config['delay'][$language['iso_code']];
			if ($language['iso_code'] == 'en')
				$carrier->delay[(int) $language['id_lang']] = $config['delay'][$language['iso_code']];
			if ($language['iso_code'] == Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')))
				$carrier->delay[(int) $language['id_lang']] = $config['delay'][$language['iso_code']];
		}

		if ($carrier->add()) {
			$groups = Group::getGroups(true);
			foreach ($groups as $group)
				Db::getInstance()->insert('carrier_group', array('id_carrier' => (int) ($carrier->id), 'id_group' => (int) ($group['id_group'])));

			$rangePrice = new RangePrice();
			$rangePrice->id_carrier = $carrier->id;
			$rangePrice->delimiter1 = '0';
			$rangePrice->delimiter2 = '10000';
			$rangePrice->add();

			$rangeWeight = new RangeWeight();
			$rangeWeight->id_carrier = $carrier->id;
			$rangeWeight->delimiter1 = '0';
			$rangeWeight->delimiter2 = '10000';
			$rangeWeight->add();

			$zones = Zone::getZones(true);
			foreach ($zones as $zone) {
				Db::getInstance()->insert('carrier_zone', array('id_carrier' => (int) ($carrier->id), 'id_zone' => (int) ($zone['id_zone'])));
				Db::getInstance()->insert('delivery', array('id_carrier' => (int) ($carrier->id), 'id_range_price' => (int) ($rangePrice->id), 'id_range_weight' => NULL, 'id_zone' => (int) ($zone['id_zone']), 'price' => '0'));
				Db::getInstance()->insert('delivery', array('id_carrier' => (int) ($carrier->id), 'id_range_price' => NULL, 'id_range_weight' => (int) ($rangeWeight->id), 'id_zone' => (int) ($zone['id_zone']), 'price' => '0'));
			}

			// Copy Logo
			if (!copy(dirname(__FILE__) . '/carrier.jpg', _PS_SHIP_IMG_DIR_ . '/' . (int) $carrier->id . '.jpg'))
				return false;

			// Return ID Carrier
			return (int) ($carrier->id);
		}

		return false;
	}

	/*
	** Global Form Config Methods
	**
	*/

	public function getContent()
	{
		$this->_html .= '<h2>' . $this->l('EMS Rate Calculator') . '</h2>';
		if (Tools::isSubmit('submit' . $this->name)) {
			$this->_postValidation();
			if (!sizeof($this->_postErrors))
				$this->_postProcess();
			else
				foreach ($this->_postErrors as $err)
					$this->_html .= '<div class="alert error"><img src="' . _PS_IMG_ . 'admin/forbbiden.gif" alt="nok" />&nbsp;' . $err . '</div>';
		}
		$this->_displayForm();
		return $this->_html;
	}

	private function _displayForm()
	{
		$this->_html .= '<fieldset>
		<legend><img src="' . $this->_path . 'logo.gif" alt="" /> ' . $this->l('EMS Module Status') . '</legend>';

		$alert = array();
		foreach ($this->configOptions as $configOption) {
			if (!Configuration::get($configOption))
				$alert['generalSettings'] = 1;
		}

		if (Db::getInstance()->getValue('SELECT COUNT(*) FROM `' . _DB_PREFIX_ . 'ems_rate_service_code` WHERE `active` = 1') < 1)
			$alert['deliveryServices'] = 1;

		if (!isset($alert['generalSettings']) && isset($alert['deliveryServices'])) {
			$this->getServiceCodes();
		}

		if (!count($alert))
			$this->_html .= '<img src="' . _PS_IMG_ . 'admin/enabled.gif" /><strong>' . $this->l('EMS Rate Calculator is configured and online!') . '</strong>';
		else {
			$this->_html .= '<img src="' . _PS_IMG_ . 'admin/warn2.png" /><strong>' . $this->l('EM Rate Calculator is not configured yet, you have to :') . '</strong>';
			$this->_html .= '<br />' . (isset($alert['generalSettings']) ? '<img src="' . _PS_IMG_ . 'admin/warn2.png" />' : '<img src="' . _PS_IMG_ . 'admin/enabled.gif" />') . ' 1) ' . $this->l('Fill in API URL, API username, API Token');
			$this->_html .= '<br />' . (isset($alert['deliveryServices']) ? '<img src="' . _PS_IMG_ . 'admin/warn2.png" />' : '<img src="' . _PS_IMG_ . 'admin/enabled.gif" />') . ' 2) ' . $this->l('Select your available Delivery Services');
		}

		$this->_html .= '</fieldset><div class="clear">&nbsp;</div>';
		$this->_html .= $this->_displayFormConfig();
	}

	public function _displayFormConfig()
	{
		$fields_form = array();

		$fields_form[0]['form'] = array(
			'legend' => array('title' => $this->l('Settings')),
			'input' => array(
				array(
					'type' => 'text',
					'label' => $this->l('API URL'),
					'name' => 'EMS_API_URL',
					'size' => 100,
					'required' => true
				),
				array(
					'type' => 'text',
					'label' => $this->l('API Username'),
					'name' => 'EMS_API_USERNAME',
					'size' => 100,
					'required' => true
				),
				array(
					'type' => 'text',
					'label' => $this->l('API Token'),
					'name' => 'EMS_API_TOKEN',
					'size' => 100,
					'required' => true
				),
				array(
					'type' => 'text',
					'label' => $this->l('Channel'),
					'name' => 'EMS_CHANNEL',
					'size' => 100
				),
				array(
					'type' => 'select',
					'multiple' => true,
					'size' => 30,
					'label' => $this->l('Shipping Services'),
					'name' => 'EMS_SERVICES[]',
					'options' => array(
						'query' => self::getServiceOptions(),
						'id' => 'id',
						'name' => 'name'
					),
				),
			),
			'submit' => array(
				'title' => $this->l('Save'),
				'class' => 'button'
			)
		);

		$helper = new HelperForm();

		$helper->module = $this;
		$helper->name_controller = $this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
		$helper->title = $this->displayName;
		$helper->show_toolbar = true;
		$helper->toolbar_scroll = true;
		$helper->submit_action = 'submit' . $this->name;
		$helper->toolbar_btn = array(
			'save' =>
			array(
				'desc' => $this->l('Save'),
				'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
					'&token=' . Tools::getAdminTokenLite('AdminModules'),
			),
			'back' => array(
				'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
				'desc' => $this->l('Back to list')
			)
		);

		foreach ($this->configOptions as $configOption) {
			$helper->fields_value[$configOption] = Configuration::get($configOption);
		}

		$selected_services = array();
		$rateServiceList = Db::getInstance()->executeS('SELECT * FROM `' . _DB_PREFIX_ . 'ems_rate_service_code` WHERE active=1');
		foreach ($rateServiceList as $service) {
			$selected_services[] = $service['id_ems_rate_service_code'];
		}
		$helper->fields_value['EMS_SERVICES[]'] = $selected_services;

		return $helper->generateForm($fields_form);
	}

	protected static function getServiceOptions()
	{
		$data = array();
		$rateServiceList = Db::getInstance()->executeS('SELECT * FROM `' . _DB_PREFIX_ . 'ems_rate_service_code`');
		foreach ($rateServiceList as $rateService) {
			$data[] = array('id' => $rateService['id_ems_rate_service_code'], 'name' => $rateService['service_name']);
		}

		return $data;
	}

	private function _postValidation()
	{
		// Check configuration values
		if (Tools::getValue('EMS_API_URL') == NULL)
			$this->_postErrors[]  = $this->l('Your API URL is not specified');
		elseif (Tools::getValue('EMS_API_USERNAME') == NULL)
			$this->_postErrors[]  = $this->l('Your API Username is not specified');
		elseif (Tools::getValue('EMS_API_TOKEN') == NULL)
			$this->_postErrors[]  = $this->l('Your API Token is not specified');
		elseif (Tools::getValue('EMS_CHANNEL') == NULL)
			$this->_postErrors[]  = $this->l('Your API Channel is not specified');
	}

	private function _postProcess()
	{
		foreach ($this->configOptions as $updateOption) {
			$val = Tools::getValue($updateOption);
			if (!empty($val)) {
				Configuration::updateValue($updateOption, $val);
			}
		}
		Db::getInstance()->update('ems_rate_service_code', array('active' => 0));
		$serviceSelected = Tools::getValue('EMS_SERVICES');

		if ($serviceSelected) {
			foreach ($serviceSelected as $ss) {
				Db::getInstance()->update('ems_rate_service_code', array('active' => 1), '`id_ems_rate_service_code` = ' . (int) ($ss));
			}
		}
	}

	public function hookupdateCarrier($params)
	{
		if ((int) ($params['id_carrier']) != (int) ($params['carrier']->id)) {
			$update = array('id_carrier' => (int) ($params['carrier']->id));
			Db::getInstance()->update('ems_rate_service_code', $update, '`id_carrier` = ' . (int) $params['id_carrier']);
		}
	}

	/*
	** Front Methods
	**
	*/
	public function getOrderShippingCost($params, $shipping_cost)
	{
		// Init var

		if (!$params->id_address_delivery) return false;
		$address = new Address($params->id_address_delivery);
		$recipient_country = Country::getNameById(Configuration::get('PS_LANG_DEFAULT'), (int) $address->id_country);
		$recipient_state = State::getNameById($address->id_state);
		$products = $params->getProducts();

		//$this->write_log(print_r($params, true));
		$currency = new Currency((int) $params->id_currency);

		// Webservices Params
		$wsParams = array(
			'id_cart' => $params->id,
			'currency' => $currency->iso_code,
			'id_address_delivery' => $params->id_address_delivery,
			'recipient_address1' => $address->address1,
			'recipient_address2' => $address->address2,
			'recipient_postalcode' => $address->postcode,
			'recipient_city' => $address->city,
			'recipient_country' => $recipient_country,
			'recipient_state' => $recipient_state,
			'products' => $params->getProducts()
		);
		$wsParams['hash'] = $this->getOrderShippingCostHash($wsParams);

		$row = Db::getInstance()->getRow("
			SELECT * FROM `" . _DB_PREFIX_ . "ems_rate_service_code`
			WHERE `id_carrier` = " . (int) $this->id_carrier . " AND `active` = 1");
		if ($row['service_code'])
			$wsParams['service_code'] = $row['service_code'];
		else
			return false;

		$wscost = 0;
		// Check cache
		$cacheList = Db::getInstance()->executeS("
			SELECT * FROM `" . _DB_PREFIX_ . "ems_cache`
			WHERE `hash` = '" . pSQL($wsParams['hash']) . "'");
		if (Db::getInstance()->NumRows()) {
			foreach ($cacheList as $v) {

				if ($v['service_code'] == $wsParams['service_code']) {
					//$this->write_log("CACHE-MATCH" . $wsParams['service_code'] . "\n");
					$wscost = $v['amount'];
				}
			}
		} else {
			$wscost = $this->getWebserviceShippingCost($wsParams);
		}

		if ($wscost > 0)
			return $wscost + $shipping_cost;
		else
			return false;
	}

	public function getOrderShippingCostExternal($params)
	{
		return $this->getOrderShippingCost($params, 0);
	}

	public function getWebserviceShippingCost($wsParams)
	{
		$to = array(
			'address1' => $wsParams['recipient_address1'],
			'city' => $wsParams['recipient_city'],
			'state' => $wsParams['recipient_state'],
			'zip' => $wsParams['recipient_postalcode'],
			'country' => $wsParams['recipient_country']
		);

		$wsParams['to'] = $to;

		$items = array();
		foreach ($wsParams['products'] as $product) {
			$width = 0;
			$height = 0;
			$depth = 0;
			$weight = 0;
			$sku = '';

			if ($product['width'] && $product['width'] > $width) $width = $product['width'];
			if ($product['height'] && $product['height'] > $height) $height = $product['height'];
			if ($product['depth'] && $product['depth'] > $depth) $depth = $product['depth'];
			if ($product['weight'])
				$weight = $product['weight'];

			$sku = $product['reference'];

			for ($i = 1; $i <= $product['quantity']; $i++) {
				$items[] = array('sku' => $sku, 'lengthInCM' => $depth, 'widthInCM' => $width, 'heightInCM' => $height, 'weightInKG' => $weight);
			}
		}
		$wsParams['items'] = $items;

		// If webservice return a cost, we add it, else, we return the original shipping cost
		$result = $this->getEMSShippingCost($wsParams);
		if ($result['connect'] && $result['cost'] > 0)
			return $result['cost'];
		return false;
	}



	/*
	** Webservices Methods
	**
	*/

	public function getEMSShippingCost($wsParams)
	{
		// Check Arguments
		if (!$wsParams)
			return array('connect' => false, 'cost' => 0);
		// Sending Request
		$data = array("to" => $wsParams['to'], "items" => $wsParams['items'], "currency" => $wsParams['currency']);
		//log debug
		// $this->write_log(print_r($data, true));

		$data_string = json_encode($data);
		$response = $this->sendRemote($this->api_url, "POST", $data_string);
		//$this->write_log(print_r($rates, true));	
		$cost = 0;
		if (count($response->rates)) {
			foreach ($response->rates as $rate_item) {
				$service_code = (string) $rate_item->service;
				$service_name = (string) $rate_item->carrier;
				$transit_time = (string) $rate_item->transitTime;
				$amount = (int) $rate_item->amount;
				if ($amount > 0 && $service_code != ''  && $service_name != '') {
					$amount = $amount / 100;
					$rate = array(
						'service_code' => $service_code,
						'service_name' => $service_name,
						'transit_time' => $transit_time,
						'service_currency' => $wsParams['currency'],
						'amount' => $amount
					);
					$this->saveOrderShippingCostCache($wsParams, $rate);
					//$this->write_log(print_r($rate, true));
					//$this->write_log("service_code=" . $wsParams['service_code'] . ", " . "service=" . $rate_item->service . "\n");
					if ($wsParams['service_code'] == $rate_item->service) {
						//$this->write_log("MATCH" . $rate_item->service . "\n");
						$cost = $amount;
					}
				}
			}
		}

		if ($cost > 0)
			return array('connect' => true, 'cost' => $cost);
		else
			return array('connect' => false, 'cost' => 0);
	}

	public function getOrderShippingCostHash($wsParams)
	{
		$paramHash = '';
		$productHash = '';
		foreach ($wsParams['products'] as $product) {
			if (!empty($productHash))
				$productHash .= '|';
			$productHash .= $product['id_product'] . ':' . $product['id_product_attribute'] . ':' . $product['cart_quantity'];
		}
		foreach ($wsParams as $k => $v)
			if ($k != 'products')
				$paramHash .= '/' . $v;
		return md5($productHash . $paramHash);
	}

	public function saveOrderShippingCostCache($wsParams, $rate)
	{
		$insert = array(
			'id_cart' => (int) ($wsParams['id_cart']),
			'hash' => pSQL($wsParams['hash']),
			'service_code' => $rate['service_code'],
			'service_name' => pSQL($rate['service_name']),
			'transit_time' => pSQL($rate['transit_time']),
			'service_currency' => pSQL($rate['service_currency']),
			'amount' => pSQL($rate['amount'])
		);
		Db::getInstance()->insert('ems_cache', $insert);
	}


	public function getServiceCodes()
	{
		$config = Configuration::getMultiple($this->configOptions);

		//shippng api info
		if (isset($config['EMS_API_URL']))
			$this->api_url = $config['EMS_API_URL'];
		if (isset($config['EMS_API_USERNAME']))
			$this->api_username = $config['EMS_API_USERNAME'];
		if (isset($config['EMS_API_TOKEN']))
			$this->api_token = $config['EMS_API_TOKEN'];
		if (isset($config['EMS_CHANNEL']))
			$this->channel = $config['EMS_CHANNEL'];


		$rates = array();

		$url  = str_replace("/query", "/services", $this->api_url);
		$services = $this->sendRemote($url, "GET", null);

		if (count($services->payload)) {
			foreach ($services->payload as $rate_item) {
				$rates[] = array(
					'service_code' => $rate_item->service,
					'service_name' => $rate_item->carrier,
					'transit_time' => ""
				);
			}
		}

		$rates = array_unique($rates, SORT_REGULAR);
		$this->write_log(print_r($rates, true));
		$this->write_log("Done service code");

		if (count($rates)) {
			foreach ($rates as $rate_item) {
				$newRow = array(
					'id_carrier' => 0,
					'service_code' => pSQL($rate_item['service_code']),
					'service_name' => pSQL($rate_item['service_name']),
					'transit_time' => pSQL($rate_item['transit_time']),
					'active' => 0
				);
				try {
					Db::getInstance()->insert('ems_rate_service_code', $newRow);
				} catch (Exception $e) {
					echo 'Message: ' . $e->getMessage();
				}
			}
		}

		//create carriers
		$this->installCarriers();
	}



	public function sendRemote($url, $method, $data_string)
	{
		$url = trim($url);
		$headers = array(
			'Content-Type: application/json',
			'Authorization: Basic ' . base64_encode($this->api_username . ':' . $this->api_token),
			'Channel: ' . $this->channel,
			'Content-Length: ' . strlen($data_string)
		);

		$ch = curl_init($url);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
		if (false == is_null($data_string)) {
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
		}
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		//The response from the API
		$response = curl_exec($ch);
		// print_r($data_string);
		// die;
		curl_close($ch);
		$rates = json_decode($response);
		//debug data
		//ems_write_log(print_r($rates, true));
		return $rates;
	}

	function write_log($message)
	{
		$fp = fopen(dirname(__FILE__) . '/log/debug.txt', 'a');
		fwrite($fp, $message);
		fclose($fp);
	}
}
